@@ -66,7 +66,9 @@ import System.Mem.StableName
 import GHC.Generics
 #endif
 
-#if __GLASGOW_HASKELL__ >= 702
+#if __GLASGOW_HASKELL__ >= 710
+import GHC.Fingerprint.Type(Fingerprint(..))
+#elif __GLASGOW_HASKELL__ >= 702
 import Data.Typeable.Internal(TypeRep(..))
 import GHC.Fingerprint.Type(Fingerprint(..))
 #endif
@@ -83,9 +85,28 @@ import Foreign.C.Types (CInt)
 import qualified Data.ByteString.Lazy.Internal as BL  -- foldlChunks
 #endif
 
+#if MIN_VERSION_bytestring(0,10,4)
+import qualified Data.ByteString.Short.Internal as BSI
+#endif
+
 #ifdef VERSION_integer_gmp
+
+# if MIN_VERSION_integer_gmp(1,0,0)
+#  define MIN_VERSION_integer_gmp_1_0_0
+# endif
+
 import GHC.Exts (Int(..))
 import GHC.Integer.GMP.Internals (Integer(..))
+# if defined(MIN_VERSION_integer_gmp_1_0_0)
+import GHC.Exts (sizeofByteArray#)
+import GHC.Integer.GMP.Internals (BigNat(BN#))
+# endif
+#endif
+
+#if MIN_VERSION_base(4,8,0)
+import Data.Void (Void, absurd)
+import GHC.Natural (Natural(..))
+import GHC.Exts (Word(..))
 #endif
 
 #include "MachDeps.h"
@@ -97,7 +118,11 @@ infixl 0 `hashWithSalt`
 
 -- | A default salt used in the implementation of 'hash'.
 defaultSalt :: Int
-defaultSalt = 0xdc36d1615b7400a4
+#if WORD_SIZE_IN_BITS == 64
+defaultSalt = -2578643520546668380  -- 0xdc36d1615b7400a4
+#else
+defaultSalt = 0x087fc72c
+#endif
 {-# INLINE defaultSalt #-}
 
 -- | The class of types that can be converted to a hash value.
@@ -233,8 +258,41 @@ instance Hashable Char where
     hash = fromEnum
     hashWithSalt = defaultHashWithSalt
 
+#if defined(MIN_VERSION_integer_gmp_1_0_0)
+instance Hashable BigNat where
+    hashWithSalt salt (BN# ba) = hashByteArrayWithSalt ba 0 numBytes salt
+                                 `hashWithSalt` size
+      where
+        size     = numBytes `quot` SIZEOF_HSWORD
+        numBytes = I# (sizeofByteArray# ba)
+#endif
+
+#if MIN_VERSION_base(4,8,0)
+instance Hashable Natural where
+# if defined(MIN_VERSION_integer_gmp_1_0_0)
+    hash (NatS# n)   = hash (W# n)
+    hash (NatJ# bn)  = hash bn
+
+    hashWithSalt salt (NatS# n)   = hashWithSalt salt (W# n)
+    hashWithSalt salt (NatJ# bn)  = hashWithSalt salt bn
+# else
+    hash (Natural n) = hash n
+
+    hashWithSalt salt (Natural n) = hashWithSalt salt n
+# endif
+#endif
+
 instance Hashable Integer where
 #if defined(VERSION_integer_gmp)
+# if defined(MIN_VERSION_integer_gmp_1_0_0)
+    hash (S# n)   = (I# n)
+    hash (Jp# bn) = hash bn
+    hash (Jn# bn) = negate (hash bn)
+
+    hashWithSalt salt (S# n)   = hashWithSalt salt (I# n)
+    hashWithSalt salt (Jp# bn) = hashWithSalt salt bn
+    hashWithSalt salt (Jn# bn) = negate (hashWithSalt salt bn)
+# else
     hash (S# int) = I# int
     hash n@(J# size# byteArray)
         | n >= minInt && n <= maxInt = fromInteger n :: Int
@@ -254,6 +312,7 @@ instance Hashable Integer where
                          `hashWithSalt` size
       where minInt = fromIntegral (minBound :: Int)
             maxInt = fromIntegral (maxBound :: Int)
+# endif
 #else
     hashWithSalt salt = foldl' hashWithSalt salt . go
       where
@@ -365,6 +424,16 @@ instance Hashable B.ByteString where
 instance Hashable BL.ByteString where
     hashWithSalt = BL.foldlChunks hashWithSalt
 
+#if MIN_VERSION_bytestring(0,10,4)
+instance Hashable BSI.ShortByteString where
+#if MIN_VERSION_base(4,3,0)
+    hashWithSalt salt sbs@(BSI.SBS ba) =
+#else
+    hashWithSalt salt sbs@(BSI.SBS ba _) =
+#endif
+        hashByteArrayWithSalt ba 0 (BSI.length sbs) salt
+#endif
+
 instance Hashable T.Text where
     hashWithSalt salt (T.Text arr off len) =
         hashByteArrayWithSalt (TA.aBA arr) (off `shiftL` 1) (len `shiftL` 1)
@@ -387,7 +456,10 @@ instance Hashable ThreadId where
 -- | Compute the hash of a TypeRep, in various GHC versions we can do this quickly.
 hashTypeRep :: TypeRep -> Int
 {-# INLINE hashTypeRep #-}
-#if __GLASGOW_HASKELL__ >= 702
+#if __GLASGOW_HASKELL__ >= 710
+-- Fingerprint is just the MD5, so taking any Int from it is fine
+hashTypeRep tr = let Fingerprint x _ = typeRepFingerprint tr in fromIntegral x
+#elif __GLASGOW_HASKELL__ >= 702
 -- Fingerprint is just the MD5, so taking any Int from it is fine
 hashTypeRep (TypeRep (Fingerprint x _) _ _) = fromIntegral x
 #elif __GLASGOW_HASKELL__ >= 606
@@ -401,6 +473,11 @@ instance Hashable TypeRep where
     hashWithSalt = defaultHashWithSalt
     {-# INLINE hash #-}
 
+#if MIN_VERSION_base(4,8,0)
+instance Hashable Void where
+    hashWithSalt _ = absurd
+#endif
+
 -- | Compute a hash value for the content of this pointer.
 hashPtr :: Ptr a      -- ^ pointer to the data to hash
         -> Int        -- ^ length, in bytes
